@@ -255,7 +255,7 @@ class RestAPI(object):
 			"customer_id":response["subscription"]["customer_id"],
 		})
 	
-	# user functions.
+	# key functions.
 	def identify_key(self, api_key=None):
 
 		# check developer success.
@@ -300,73 +300,12 @@ class RestAPI(object):
 			return r3sponse.error_response(f"API Key [{api_key}] is not authorized.")
 
 		#
-	def verify_rate_limit(self, 
-		# required.
-		api_key=None, 
-		# optional to increase speed.
-		# the uid from the api key.
-		uid=None,
-		# the plan from the api key.
-		plan=None,
-	):
-
-		# check info.
-		if plan == None or uid == None:
-			response = self.identify_key(api_key)
-			if response["error"] != None: return response
-			plan = response["plan"]
-			uid = response["uid"]
-
-		# pro / developer.
-		if self.__plans__[plan]["rate_limit"] in [None, False]:
-			return r3sponse.success_response("Successfully verified the rate limit.")
-
-		# load data.
-		response = self.firebase.users.load_data(uid)
-		if response["error"] != None: return response
-		data = response["data"]
-		timestamp = response["data"]["timestamp"]
-
-		# check timestamp.
-		date = Formats.Date()
-		success = False
-		if timestamp == None:
-			success = True
-			data["timestamp"] = date.date
-			data["requests"] = 0
-		else:
-			altered = date.increase(timestamp, days=self.__plans__[plan]["rate_reset"], format="%d-%m-%y")
-			#decreased_timestamp = date.from_seconds(decreased)
-			if date.compare(altered, date.date, format="%d-%m-%y") in ["present", "past"]:
-				success = True
-				data["timestamp"] = date.date
-				data["requests"] = 0
-
-			# check rate limit.
-			if not success and int(data["requests"]) <= self.__plans__[plan]["rate_limit"]: 
-				success = True
-
-		# response.
-		if success:
-			data["requests"] += 1
-			response = self.firebase.users.save_data(uid, data)
-			if response["error"] != None: return response
-			return r3sponse.success_response("Successfully verified the rate limit.")
-		else:
-			if plan in ["free"]:
-				return r3sponse.error_response("You have exhausted your monthly rate limit. Upgrade your membership to premium or pro for more requests.")
-			elif plan in ["premium"]:
-				return r3sponse.error_response("You have exhausted your monthly rate limit. Upgrade your membership to pro for unlimited requests.")
-			else:
-				return r3sponse.error_response("You have exhausted your monthly rate limit.")
-
-		# 
 	def get_key_by_uid(self, uid):
 		api_key = None
 		try:
 			api_key = self.uid_api_keys[uid]
 		except KeyError:
-			api_key == None
+			api_key = None
 		if api_key != None:
 			return r3sponse.success_response(f"Successfully found the uid for the specified user [{uid}].", {"api_key":self.uid_api_keys[uid]})
 		else:
@@ -376,6 +315,8 @@ class RestAPI(object):
 			if str(_api_key_) == str(api_key):
 				return r3sponse.success_response("Successfully found the uid for the specified api key.", {"uid":uid})
 		return r3sponse.error_response("Failed to find the uid for the specified api key.")
+
+	# user functions.
 	def synchronize_users(self, silent=False):
 		"""
 			Checks all users & inserts new data.
@@ -473,7 +414,110 @@ class RestAPI(object):
 		return r3sponse.success_response("Successfully synchronized the users.", log_level=log_level)
 
 		#
-	
+
+	# requests & rate limit functions.
+	def verify_rate_limit(self, 
+		# required.
+		api_key=None, 
+		# optional to increase speed.
+		# the uid from the api key.
+		uid=None,
+		# the plan from the api key.
+		plan=None,
+	):
+
+		# check info.
+		if plan == None or uid == None:
+			response = self.identify_key(api_key)
+			if response["error"] != None: return response
+			plan = response["plan"]
+			uid = response["uid"]
+
+		# pro / developer.
+		if self.__plans__[plan]["rate_limit"] in [None, False]:
+			return r3sponse.success_response("Successfully verified the rate limit.")
+
+		# load data.
+		response = self.firebase.users.load_data(uid)
+		if response["error"] != None: return response
+		data = response["data"]
+		timestamp = response["data"]["timestamp"]
+
+		# check timestamp.
+		date = Formats.Date()
+		success = False
+		if timestamp == None:
+			success = True
+			data["timestamp"] = date.date
+			data["requests"] = 0
+		else:
+			altered = date.increase(timestamp, days=self.__plans__[plan]["rate_reset"], format="%d-%m-%y")
+			#decreased_timestamp = date.from_seconds(decreased)
+			if date.compare(altered, date.date, format="%d-%m-%y") in ["present", "past"]:
+				success = True
+				data["timestamp"] = date.date
+				data["requests"] = 0
+
+			# check rate limit.
+			if not success and int(data["requests"]) <= self.__plans__[plan]["rate_limit"]: 
+				success = True
+
+		# response.
+		if success:
+			data["requests"] += 1
+			response = self.firebase.users.save_data(uid, data)
+			if response["error"] != None: return response
+			return r3sponse.success_response("Successfully verified the rate limit.")
+		else:
+			if plan in ["free"]:
+				return r3sponse.error_response("You have exhausted your monthly rate limit. Upgrade your membership to premium or pro for more requests.")
+			elif plan in ["premium"]:
+				return r3sponse.error_response("You have exhausted your monthly rate limit. Upgrade your membership to pro for unlimited requests.")
+			else:
+				return r3sponse.error_response("You have exhausted your monthly rate limit.")
+
+		# 
+	def give_free_requests(self, uid, requests=100):
+
+		# load user data.
+		reference = f"users/{uid}"
+		response = self.firebase.firestore.load(reference) 
+		if not r3sponse.success(response): return response
+		data = response["data"]
+
+		# edit data.
+		data["requests"] = -1*requests
+		data["timestamp"] = Formats.Date().date
+
+		# save data.
+		response = self.firebase.firestore.save(reference, data) 
+		if response["error"] != None: return response
+
+		# handlers.
+		return r3sponse.success_response(f"Successfully gave user [{uid}] {requests} free requests.")
+		
+		#
+	def reset_requests(self, uid):
+
+		# load user data.
+		reference = f"users/{uid}"
+		response = self.firebase.firestore.load(reference) 
+		if not r3sponse.success(response): return response
+		data = response["data"]
+
+		# edit data.
+		data["requests"] = 0
+		data["timestamp"] = Formats.Date().date
+
+		# save data.
+		response = self.firebase.firestore.save(reference, data) 
+		if response["error"] != None: return response
+
+		# handlers.
+		return r3sponse.success_response(f"Successfully gave user [{uid}] {requests} free requests.")
+		
+		#
+
 	# system functions.
 	def __save_new_user__(self, uid):
 		
@@ -484,11 +528,13 @@ class RestAPI(object):
 		# generate api key.
 		response = self.__generate_key__()
 		if response["error"] != None: return response
-		data["api_key"] = response["api_key"]
+		api_key = response["api_key"]
+		data["api_key"] = api_key
 
 		# add to self variables.
-		self.uid_api_keys[uid] = data["api_key"]
-		self.__plans__["free"]["api_keys"].append(data["api_key"])
+		self.uid_api_keys[uid] = api_key
+		if api_key not in self.__plans__["free"]["api_keys"]:
+			self.__plans__["free"]["api_keys"].append(api_key)
 
 		# save new data.
 		response = self.firebase.firestore.save(reference, data) 
